/*	$Id: choice.c,v 1.13 2001/04/26 03:23:19 pvalchev Exp $	*/

/*
 * Copyright (c) 1998-2001 Peter V. Valchev <pvalchev@gmx.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *    
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission
 *    from the author.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "phyz.h"
#include "choice.h"
#include "keys.h"

#define _PHYZ_CHOICE_C

int old_choice = -1;
int new_choice = 0;
char dilemma;

void theory_choice ();
void solver_choice ();

const CHOICE choice[] =
{
	{ "Theory", theory_choice },
	{ "Solver", solver_choice }
};

#define MAX_OPTIONS ((sizeof(choice)/sizeof(choice[0])))

/* Draw the choice menu: */

extern void
pick_choice(int old_choice, int new_choice) {
	WINDOW *wbl;
	register size_t j;
	wbl = newwin(LINES, 0, 0, 0);
	if (wbl == NULL ) {
		endwin();
		exit(1);	/* something is wrong */
	}

	attrset(A_NORMAL);
	refresh();
	move(1, 1);
	clrtobot();
	standout();
	printw("Make up your mind and select a choice:");
	standend();
	for( j = 0; j < MAX_OPTIONS; j++ ) {
		mvaddstr(3+j, 7, choice[j].text);
	}
	if (old_choice != (-1))
		mvaddstr(3+old_choice, 7, choice[old_choice].text);
	attrset(A_REVERSE);
	mvaddstr(3+new_choice, 7, choice[new_choice].text);
	attrset(A_NORMAL);
	refresh();

}	/* End of pick_choice() */

void
choice_switch(WINDOW *win) {
	Lastkey = getch();
	switch(Lastkey)
	{
	case 10: case 13:
	case 'x': case 'X':
	case KEY_ENTER:
	  erase();
	  refresh();
	  (*choice[new_choice].function)(win);
	  erase();
	  pick_choice(old_choice, new_choice);
	  break;
	case 'k':
	case KEY_UP:
	  new_choice = (new_choice == 0) ? new_choice : new_choice-1;
	  pick_choice(old_choice, new_choice);
	  break;
	case 'j':
	case KEY_DOWN:
	  new_choice = (new_choice == MAX_OPTIONS - 1) ? new_choice : new_choice + 1;
	  flist:
	  pick_choice(old_choice, new_choice);
	  break;
	case 'q': case 'Q':
	  standend();
	  endwin();
	  exit(0);
	  break;
	case 'c': case 'C':
	  CALCULATOR;
	  goto flist;
	default:
	  break;
	}
}


void theory_choice() 
{
	dilemma = '3';
}

void solver_choice()
{
	dilemma = '4';
}

/* EOF */
