/*	$Id: circular.c,v 1.24 2001/04/26 03:23:19 pvalchev Exp $	*/

/*
 * Copyright (c) 1998-2001 Peter V. Valchev <pvalchev@gmx.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *    
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission
 *    from the author.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define _PHYZ_CIRCULAR_C

#include "phyz.h"
#include "keys.h"
#include "choice.h"

#ifndef lint
static const char rcsid[] = "$Id: circular.c,v 1.24 2001/04/26 03:23:19 pvalchev Exp $";
#endif /* lint */

void vr_shmok ();
void rf_shmok ();
void vt_shmok ();
void mv_shmok ();
void m4_shmok ();
void pick_function_list ();
void function_list ();

int old_shmok = -1;
int new_shmok = 0;

char fshmok;

/* XXX: Pi (0x1c) is displayed as  (0xf1) because
 * most of the terminals lack the ability to display
 * the actual greek letter. (vt100 && vt220 are on
 * the top of my head). Ideas here'd be appreciated.
 */

#define PI_CHAR 0x1c

CHOICE flist_circular[] =
{
	{ "a = V/R", vr_shmok },
	{ "a = 2Rf", rf_shmok },
	{ "V = 2R/T", vt_shmok },
	{ "F = mV/R", mv_shmok },
	{ "F = m4Rf", m4_shmok }
};

void
circular_motion (WINDOW *win) {
	char buf[80];
	float num;
	float V, R, a, f, T, F;
	float m = 0;
	float pi = M_PI;
	char sf;
	pick_choice(old_choice, new_choice);
	while ( dilemma != '3' && dilemma != '4' ) {
		choice_switch();
	}
	win = newwin(LINES, 0, 0, 0);

    if ( dilemma == '3' ) {
	mvwprintw(win, 1, 2, "Circular Motion:\n");
	mvwprintw(win, 2, 2, "Uniform Circular Motion - objects travelling in a circle at a constant speed\n  V = d/t\n  V = (2**R)/t\n");
	mvwprintw(win, 5, 2, "Period (T) - time to complete one trip around the circle; measured in seconds/minutes/hours/days/years:");
	mvwprintw(win, 7, 2, "  V = (2R)/T");
	waddstr(win, "\nRemember: T = 1/f\tFrequency (f) - number of trips completed in one second, measured in Hertz (Hz), kHz, MHz");
	waddstr(win, "\n  V = 2Rf");
	waddstr(win, "\nCentripetal Acceleration:");
	waddstr(win, "\n\n  When an object travels in a circle its speed remains constant but its direction is constantly changing. Since velocity contains both magnitude (speed) and direction, the velocity is not constant for an object travelling in a circle - it is always TANGENT to the circle.");
	waddstr(win, "\n  Since velocity changes with time, acceleration must occur. The acceleration is directed towards the center and is called Centripetal Acceleration");
	waddstr(win, "\n  a = V/R");
	waddstr(win, "\n  a = (4R)/T or a = 4Rf");
	wrefresh(win);
	Continue(win);
	werase(win);
	wrefresh(win);
	waddstr(win, "\t\nCentripetal Force ... the force that causes centripetal acceleration. Unique in that it only causes a change in direction of the object, not the magnitude (speed).\n");
	waddstr(win, "  F = ma	(a = centripetal acceleration)\n");
	waddstr(win, "Therefore using the known acceleration/speed formulas:\n");
	waddstr(win, "  F = mV/R\n");
	waddstr(win, "  F = 4mR/T\n");
	waddstr(win, "  F = 4mRf\n");
	Continue(win);
	werase(win);
	wrefresh(win);
	waddstr(win, "\t\nUNBANKED CURVES\n");
	waddstr(win, "\tIn order for a car to round an unbanked curve the frictional force of the tires against the road must provide the centripetal force to turn the car.\n");
	waddstr(win, "  Ff = Fc\n");
	waddstr(win, "where Ff = Fn and Fc = mV/R  (Fn is the normal force, opposite to Fg = mg)\n"); 
	waddstr(win, "\nExample:\n");
	waddstr(win, "How fast can a 1200 kg car round an unbanked curve of radius 62 m if the coefficient of friction between the tires and the road is 0.44?\n");
	waddstr(win, "\nSolution:\n");
	waddstr(win, "  Ff = Fc\n  F = Fg = 1200 kg x 9.81 m/s = 11772 N\n");
	waddstr(win, "  Ff =  x Fn = 0.44 x 11772 N = 5179.68 N\n");
	waddstr(win, "  Ff = Fc = mV/R\n  V = (R x Ff)/m\n  V = 16.4 m/s\n");
	Continue(win);
	werase(win);
	wrefresh(win);
	waddstr(win, "\t\nVERTICAL CIRCLES\n
In vertical circles .. the centripetal force may vary from point to point.
F net = F(up) + F(down)
F net is Fc (centripetal force)
at the top   _o_   
            /   \\    Fc = T + Fg, where T is the tension in the string
	   |     |                           or 'the reaction force'.
	    \\___/

at the bottom  ___   
              /   \\    Fc = T - Fg, where T is the tension in the string
             |     |                           or 'the reaction force'.
              \\_o_/
	");
	Continue(win);
	} 

    if ( dilemma == '4' ) { 
	werase(win);
	wrefresh(win);
	*buf = 0;
	num = 0;
	/* XXX: new menu */
	pick_function_list(old_choice, new_choice);
	/* listen until : */
	while ( fshmok != '1' && fshmok != '2' && fshmok != '3' && fshmok != '4' && fshmok != '5' ) {
		function_list();
	}
	echo();
	if ( fshmok == '1' ) {
		werase(win);
		wrefresh(win);
		waddstr(win, "\na = V/R");
		waddstr(win, "\nSolve for (enter the letter)?: ");
		wscanw(win, "%s", &sf, buf);
			if ( !strncmp(&sf, "a", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "Speed? V = ");
				wscanw(win, "%f", &num, buf);
				V = num;
				num = 0;
				wprintw(win, "V = %f", V);
				wprintw(win, "\nRadius? R = ");
				wscanw(win, "%f", &num, buf);
				R = num;
				num = 0;
				wprintw(win, "R = %f", R);
				num = (V*V)/R;
				wprintw(win, "\t\nAcceleration = a = %f", num);
			} else if ( !strncmp(&sf, "V", sizeof(char)) || !strncmp(&sf, "v", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "What's the acceleration? a = ");
				wscanw(win, "%f", &num, buf);
				a = num; 
				num = 0;
				wprintw(win, "a = %f", a);
				wprintw(win, "\nRadius? R = ");
				wscanw(win, "%f", &num, buf);
				R = num;
				num = 0;
				wprintw(win, "R = %f", R);
				num = sqrt(R*a);
				wprintw(win, "\t\nSpeed = V = %f", num);
			} else if ( !strncmp(&sf, "R", sizeof(char)) || !strncmp(&sf, "r", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "What's the acceleration? a = ");
				wscanw(win, "%f", &num, buf);
				a = num;
				num = 0;
				wprintw(win, "a = %f", a);
				wprintw(win, "\nSpeed? V = ");
				wscanw(win, "%f", &num, buf);
				V = num;
				num = 0;
				wprintw(win, "V = %f", V);
				num = (V*V)/a;
				wprintw(win, "\t\nDistance = R = %f", num);
			} else {
				NOPE;
			}
		Continue(win);
	} else if ( fshmok == '3' ) {
		erase(); 
		refresh();
		addstr("\nV = 2R/T");
		addstr("\nSolve for (enter the letter)?: ");
		scanw("%s", &sf, buf);
		if ( !strncmp(&sf, "V", sizeof(char)) || !strncmp(&sf, "v", sizeof(char))) {
			SOLVE_FOR;
			wprintw(win, "What's the radius? R = ");
			wscanw(win, "%f", &num, buf);
			R = num; num = 0;
			wprintw(win, "R = %f", R);
			wprintw(win, "\nPeriod? T = ");
			wscanw(win, "%f", &num, buf);
			T = num; num = 0;
			wprintw(win, "T = %f", T);
			num = (2*pi*R)/T;
			wprintw(win, "\nSpeed = V = %f", num);
			num = 0;
		} else if ( !strncmp(&sf, "R", sizeof(char)) || !strncmp(&sf, "r", sizeof(char))) 
		{
			SOLVE_FOR;
			wprintw(win, "What's the speed? V = ");
			wscanw(win, "%f", &num, buf);
			V = num; num = 0;
			wprintw(win, "V = %f", V);
			wprintw(win, "\nPeriod? T = ");
			wscanw(win, "%f", &num, buf);
			T = num; num = 0;
			wprintw(win, "T = %f", T);
			num = (V*T)/(2*pi);
			wprintw(win, "\nRadius = R = %f", num);
			num = 0;
		} else if ( !strncmp(&sf, "T", sizeof(char)) || !strncmp(&sf, "t", sizeof(char))) 
		{
			SOLVE_FOR;
			wprintw(win, "What's the radius? R = ");
			wscanw(win, "%f", &num, buf);
			R = num; num = 0;
			wprintw(win, "R = %f", R);
			wprintw(win, "\nSpeed? V = ");
			wscanw(win, "%f", &num, buf);
			V = num; num = 0;
			wprintw(win, "V = %f", V);
			num = (2*pi*R)/V;
			wprintw(win, "\nPeriod = T = %f", num);
			num = 0;
		} else { 
			NOPE;
		} 
		Continue(win);
	} else if ( fshmok == '4' ) {
		erase(); 
		refresh();
		addstr("\nF = mV/R");
		addstr("\nSolve for (enter the letter)?: ");
		scanw("%s", &sf, buf);
		if ( !strncmp(&sf, "F", sizeof(char)) || !strncmp(&sf, "f", sizeof(char))) {
			SOLVE_FOR;
			wprintw(win, "What's the mass? m = ");
			wscanw(win, "%f", &num, buf);
			m = num; num = 0;
			wprintw(win, "m = %f", m);
			wprintw(win, "\nSpeed? V = ");
			wscanw(win, "%f", &num, buf);
			V = num; num = 0;
			wprintw(win, "V = %f", V);
			wprintw(win, "\nRadius? \nR = ");
			wscanw(win, "%f", &num, buf);
			R = num; num = 0;
			wprintw(win, "R = %f", R);
			num = (m*V*V)/R;
			wprintw(win, "\nCentripetal Force = Fc = %f", num);
			num = 0;
		} else if ( !strncmp(&sf, "V", sizeof(char)) || !strncmp(&sf, "v", sizeof(char))) 
		{
			SOLVE_FOR;
			wprintw(win, "What's the centripetal force? F = ");
			wscanw(win, "%f", &num, buf);
			F = num; num = 0;
			wprintw(win, "F = %f", F);
			wprintw(win, "\nRadius? R = ");
			wscanw(win, "%f", &num, buf);
			R = num; num = 0;
			wprintw(win, "R = %f", R);
			wprintw(win, "\nMass? \nm = ");
			wscanw(win, "%f", &num, buf);
			m = num; num = 0;
			wprintw(win, "m = %f", m);
			num = sqrt((F*R)/m);
			wprintw(win, "\nSpeed = V = %f", num);
			num = 0;
		} else if ( !strncmp(&sf, "R", sizeof(char)) || !strncmp(&sf, "r", sizeof(char))) 
		{
			SOLVE_FOR;
			wprintw(win, "What's the centripetal force? F = ");
			wscanw(win, "%f", &num, buf);
			F = num; num = 0;
			wprintw(win, "F = %f", F);
			wprintw(win, "\nSpeed? V = ");
			wscanw(win, "%f", &num, buf);
			V = num; num = 0;
			wprintw(win, "V = %f", V);
			wprintw(win, "\nMass? \nm = ");
			wscanw(win, "%f", &num, buf);
			m = num; num = 0;
			wprintw(win, "m = %f", m);
			num = F/(m*V*V);
			wprintw(win, "\nRadius = R = %f", num);
			num = 0;
		} else if ( !strncmp(&sf, "m", sizeof(char))) { 
			SOLVE_FOR;
			wprintw(win, "What's the centripetal force? F = ");
			wscanw(win, "%f", &num, buf);
			F = num; num = 0;
			wprintw(win, "F = %f", F);
			wprintw(win, "\nSpeed? V = ");
			wscanw(win, "%f", &num, buf);
			V = num; num = 0;
			wprintw(win, "V = %f", V);
			wprintw(win, "\nRadius? \nR = ");
			wscanw(win, "%f", &num, buf);
			R = num; num = 0;
			wprintw(win, "R = %f", R);
			num = (F*R)/(V*V);
			wprintw(win, "\nMass = m = %f", num);
			num = 0;
		} else { 
			NOPE;
		} 
		Continue(win);
	} else if ( fshmok == '2' ) {
		werase(win);
		wrefresh(win);
		waddstr(win, "\na = 2Rf");
		waddstr(win, "\nSolve for (enter the letter)?: ");
		wscanw(win, "%s", &sf, buf);
			if ( !strncmp(&sf, "a", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "What's the radius? R = ");
				wscanw(win, "%f", &num, buf);
				R = num;
				num = 0;
				wprintw(win, "R = %f", R);
				wprintw(win, "\nFrequency? f = ");
				wscanw(win, "%f", &num, buf);
				f = num;
				num = 0;
				wprintw(win, "f = %f", f);
				num = 2*pi*R*f;
				wprintw(win, "\nAcceleration = a = %f", num);
				num = 0;
			} else if ( !strncmp(&sf, "R", sizeof(char)) || !strncmp(&sf, "r", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "What's the acceleration? a = ");
				wscanw(win, "%f", &num, buf);
				a = num; num = 0;
				wprintw(win, "a = %f", a);
				wprintw(win, "\nFrequency? f = ");
				wscanw(win, "%f", &num, buf);
				f = num; num = 0;
				wprintw(win, "f = %f", f);
				num = a/(2*pi*f);
				wprintw(win, "\nRadius = R = %f", num);
				num = 0;
			} else if ( !strncmp(&sf, "f", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "What's the acceleration? a = ");
				wscanw(win, "%f", &num, buf);
				a = num;
				num = 0;
				wprintw(win, "a = %f", a);
				wprintw(win, "\nRadius? R = ");
				wscanw(win, "%f", &num, buf);
				R = num; num = 0;
				wprintw(win, "R = %f", R);
				num = a/(2*pi*R);
				wprintw(win, "\nFrequency = f = %f", num);
				num = 0;
			} else {
				NOPE;
			}
		Continue(win);
	} else if ( fshmok == '5' ) {
		erase(); 
		refresh();
		addstr("\nF = m4Rf");
		addstr("\nSolve for (enter the letter)?: ");
		scanw("%s", &sf, buf);
		if ( !strncmp(&sf, "F", sizeof(char)) ) {
			SOLVE_FOR;
			wprintw(win, "What's the mass? m = ");
			wscanw(win, "%f", &num, buf);
			m = num; num = 0;
			wprintw(win, "m = %f", m);
			wprintw(win, "\nFrequency? f = ");
			wscanw(win, "%f", &num, buf);
			f = num; num = 0;
			wprintw(win, "f = %f", f);
			wprintw(win, "\nRadius? \nR = ");
			wscanw(win, "%f", &num, buf);
			R = num; num = 0;
			wprintw(win, "R = %f", R);
			num = m*4*pi*pi*R*f*f;
			wprintw(win, "\nCentripetal Force = Fc = %f", num);
			num = 0;
		} else if ( !strncmp(&sf, "R", sizeof(char)) || !strncmp(&sf, "r", sizeof(char))) 
		{
			SOLVE_FOR;
			wprintw(win, "What's the centripetal force? F = ");
			wscanw(win, "%f", &num, buf);
			F = num; num = 0;
			wprintw(win, "F = %f", F);
			wprintw(win, "\nFrequency? f = ");
			wscanw(win, "%f", &num, buf);
			f = num; num = 0;
			wprintw(win, "V = %f", f);
			wprintw(win, "\nMass? \nm = ");
			wscanw(win, "%f", &num, buf);
			m = num; num = 0;
			wprintw(win, "m = %f", m);
			num = F/(m*4*pi*pi*f*f);
			wprintw(win, "\nRadius = R = %f", num);
			num = 0;
		} else if ( !strncmp(&sf, "m", sizeof(char))) { 
			SOLVE_FOR;
			wprintw(win, "What's the centripetal force? F = ");
			wscanw(win, "%f", &num, buf);
			F = num; num = 0;
			wprintw(win, "F = %f", F);
			wprintw(win, "\nFrequency? f = ");
			wscanw(win, "%f", &num, buf);
			f = num; num = 0;
			wprintw(win, "f = %f", f);
			wprintw(win, "\nRadius? \nR = ");
			wscanw(win, "%f", &num, buf);
			R = num; num = 0;
			wprintw(win, "R = %f", R);
			num = F/(m*4*pi*pi*R*f*f);
			wprintw(win, "\nMass = m = %f", num);
			num = 0;
		} else if ( !strncmp(&sf, "f", sizeof(char))) {
			SOLVE_FOR;
			wprintw(win, "What's the centripetal force? F = ");
			wscanw(win, "%f", &num, buf);
			F = num; num = 0;
			wprintw(win, "F = %f", F);
			wprintw(win, "\nMass? f = ");
			wscanw(win, "%f", &num, buf);
			m = num; num = 0;
			wprintw(win, "f = %f", m);
			wprintw(win, "\nRadius? \nR = ");
			wscanw(win, "%f", &num, buf);
			R = num; num = 0;
			wprintw(win, "R = %f", R);
			num = sqrt(F/(m*4*pi*pi*R));
			wprintw(win, "\nFrequency = f = %f", num);
			num = 0;
		} else { 
			NOPE;
		} 
		Continue(win);
    	/* end of elif dilemma ... */
	} else {
		werase(win);
		wrefresh(win);
		wprintw(win, "\nDoesn't exist.");
		Continue(win);
	}
	} 

	dilemma = '0';
	fshmok = '0';
}

void
pick_function_list(int old_choice, int new_choice) {
#define MAX_OPTIONS ((sizeof(flist_circular)/sizeof(flist_circular[0])))
	register size_t f;
	attrset(A_NORMAL);
	refresh();
	move(1, 1);
	clrtobot();
	standout();
	printw("Solve the Problem. Select a formula to use.\n\n");
	standend();
 	printw("These formulas are available for use; enter the letter of the variable you want to solve for when asked. Carrying appropriate units is your responsibility!");
	for ( f = 0; f < MAX_OPTIONS; f++ ) {
		mvaddstr(6+f, 7, flist_circular[f].text);
	}
	if (old_choice != (-1)) 
		mvaddstr(6+old_choice, 7, flist_circular[old_choice].text);
	attrset(A_REVERSE);
	mvaddstr(6+new_shmok, 7, flist_circular[new_shmok].text);
	attrset(A_NORMAL);
	mvaddstr(LINES-2, 1, "Use 'j' for down and 'k' for up, Enter to run.");
	refresh();
}

void vr_shmok()
{ 
	fshmok = '1';
}

void rf_shmok()
{
	fshmok = '2';
}

void vt_shmok() 
{
	fshmok = '3';
}

void mv_shmok() {
	fshmok = '4';
}

void m4_shmok() {
	fshmok = '5';
}

void function_list(WINDOW *win) {
	Lastkey = getch();
	switch(Lastkey)
	{
	case 10: case 13:
	case 'x': case 'X':
	case KEY_ENTER:
		erase();
		refresh();
		(*flist_circular[new_shmok].function)(win);
		erase();
		pick_function_list(old_shmok, new_shmok);
		break;
	case 'k':
	case KEY_UP:
		new_shmok = (new_shmok == 0) ? new_shmok : new_shmok-1;
		pick_function_list(old_shmok, new_shmok);
		break;
	case 'j':
	case KEY_DOWN:
		new_shmok = (new_shmok == MAX_OPTIONS - 1) ? new_shmok : new_shmok + 1;
		pick_function_list(old_shmok, new_shmok);
		break;
	case 'q': case 'Q':
		standend();
		endwin();
		exit(0);
		break;
	default:
		break;
	}
}

/* EOF */
