/*	$Id: energy.c,v 1.2 2001/04/26 03:23:19 pvalchev Exp $	*/

/*
 * Copyright (c) 1998-2001 Peter V. Valchev <pvalchev@gmx.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *    
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission
 *    from the author.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define _PHYZ_ENERGY_C

#include "phyz.h"
#include "keys.h"
#include "choice.h"

#ifndef lint
static const char rcsid[] = "$Id: energy.c,v 1.2 2001/04/26 03:23:19 pvalchev Exp $";
#endif /* lint */

void mgh_shmeg(), ek_shmeg(), kx_shmeg();
void pick_function_list_eg ();
void function_list_eg ();

int old_shmeg = -1;
int new_shmeg = 0;

char fshmeg;

CHOICE flist_energy[] =
{
	{ "Ep = mgh", mgh_shmeg },
	{ "Ek = mV/2", ek_shmeg },
	{ "Ep = kx/2 -- for compressed strings", kx_shmeg }
};

extern void
energy (WINDOW *win) {
	char buf[80];
	char sf;
	float g = 9.81;
	float h, E, k, x, V, num;
	float m = 0;
	pick_choice(old_choice, new_choice);
	while ( dilemma != '3' && dilemma != '4' ) {
		choice_switch();
	}
	win = newwin(LINES, 0, 0, 0);

	/* Theory */
    if ( dilemma == '3' ) {
	waddstr(win, "\t\nENERGY\n
ENERGY definition: the ability to do work
ENERGY units: the same as for WORK, therefore Energy is measured in JOULES.

There are many forms of energy... Mechanical energy, Thermal energy, Electrical energy, Nuclear energy, etc.

When energy is changed from one form to another.. work is done.. 

Work is the change of energy.

Energy is _scalar_.
	");

	Continue(win);
	werase(win);
	wrefresh(win);
	waddstr(win, "\n\tMechanical Energy\n
Mechanical energy is the energy of an object because of its position (POTENTIAL ENERGY) or because of its motion (KINETIC ENERGY).

A) POTENTIAL ENERGY
Potential energy is also known as energy of POSITION or STORED Energy.. It is equal to the WORK done to put the object into that POSITION.


If the work was done against gravity, then the Potential Energy is referred to as GRAVITATIONAL Potential Energy.

For example:
	If the work is done to raise an object above a table, then the work is stored and is recovarable.  If the object falls to the table it has the capacity to do WORK on the table (such as changing of its shape).

	");

	Continue(win);
	werase(win);
	wrefresh(win);
	waddstr(win, "
If the work is done against a bow string or compressing a string.. the potential energy is referred to as Elastic Potential Energy.
Example:
An archer draws an arrow back by exerting a force of 90 N on the string.  If the string is drawn back 80 cm, what is the Potential Energy (Elastic) of the bow string?
Solution:
Ep = WORK = Fa x d = 90 N * 0.8 m = 72 J
The bow string has the potential to do 72 J of work on the arrow when released.

Also REMEMBER:
Energy is conserved theoretically.  However, when friction is involved .. the force of friction is not recoverable.. it is converted into Thermal Energy (HEAT).

	");
	Continue(win);
	werase(win);
	wrefresh(win);
	waddstr(win, "
B) KINETIC ENERGY
Kinetic Energy is the energy of MOTION...
Work is done on an object when a FORCE acts through a DISTANCE
	W = Fd
A force applied to an object will cause a MASS to accelerate:
	Since F = ma
	=> W = mad
	But V = Vo + 2ad and when Vo = 0 => V = 2ad <=> ad = V/2
	Thus WORK = mad becomes W = mV/2
	Or Work = mV/2

The work done in accelerating an object from rest is called Kinetic Energy:
	thus Ek = mV/2
	");
	Continue(win);
	} 

	/* Solver */
    if ( dilemma == '4' ) { 
	werase(win);
	wrefresh(win);
	*buf = 0;
	num = 0;
	/* XXX: new menu */
	pick_function_list_eg(old_choice, new_choice);
	/* listen until : */
	while ( fshmeg != '1' && fshmeg != '2' && fshmeg != '3' ) {
		function_list_eg();
	}
	echo();
	if ( fshmeg == '1' ) {
		werase(win);
		wrefresh(win);
		waddstr(win, "\nE = mgh");
		waddstr(win, "\nSolve for (enter the letter)?: ");
		wscanw(win, "%s", &sf, buf);
			if ( !strncmp(&sf, "E", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "Mass? m = ");
				SW; m = num; num = 0;
				wprintw(win, "m = %f", m);
				wprintw(win, "\nHeight? h = ");
				SW; h = num; num = 0;
				wprintw(win, "h = %f", h);
				num = m*g*h;
				wprintw(win, "\nPotential Energy = Ep = %f", num);
			} else if ( !strncmp(&sf, "m", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "Potential Energy? E = ");
				SW; E = num; num = 0;
				wprintw(win, "E = %f", E);
				wprintw(win, "\nHeight? h = ");
				SW; h = num; num = 0;
				wprintw(win, "h = %f", h);
				num = E/(g*h);
				wprintw(win, "\nMass = m = %f", num);
			} else if ( !strncmp(&sf, "h", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "Potential Energy? E = ");
				SW; E = num; num = 0;
				wprintw(win, "E = %f", E);
				wprintw(win, "\nHeight? h = ");
				SW; h = num; num = 0;
				wprintw(win, "h = %f", h);
				num = E/(m*g);
				wprintw(win, "\nHeight = h = %f", num);
			} else if ( !strncmp(&sf, "g", sizeof(char)) ) {
				wprintw(win, "\nYou're asking me to do the impossible.");
				wprintw(win, "\ng is a constant. Sorry.");
			} else {
				NOPE;
			}
		Continue(win);
	} else if ( fshmeg == '2' ) {
		werase(win);
		wrefresh(win);
		waddstr(win, "\nE = mV/2");
		waddstr(win, "\nSolve for (enter the letter)?: ");
		wscanw(win, "%s", &sf, buf);
			if ( !strncmp(&sf, "E", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "Mass? m = ");
				SW; m = num; num = 0;
				wprintw(win, "m = %f", m);
				wprintw(win, "\nSpeed? V = ");
				SW; V = num; num = 0;
				wprintw(win, "V = %f", V);
				num = (m*V*V)/2;
				wprintw(win, "\nKinetic Energy = Ek = %f", num);
			} else if ( !strncmp(&sf, "m", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "Kinetic Energy? Ek = ");
				SW; E = num; num = 0;
				wprintw(win, "E = %f", E);
				wprintw(win, "\nSpeed? V = ");
				SW; V = num; num = 0;
				wprintw(win, "V = %f", V);
				num = (2*E)/(V*V);
				wprintw(win, "\nMass = m = %f", num);
			} else if ( !strncmp(&sf, "V", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "Kinetic Energy? Ek = ");
				SW; E = num; num = 0;
				wprintw(win, "E = %f", E);
				wprintw(win, "\nMass? m = ");
				SW; m = num; num = 0;
				wprintw(win, "m = %f", m);
				num = sqrt((2*E)/m);
				wprintw(win, "\nSpeed = V = %f", num);
			} else {
				NOPE;
			}
		Continue(win);	
	} else if ( fshmeg == '3' ) {
		werase(win); 
		wrefresh(win);
		waddstr(win, "\nEp = kx/2");
		waddstr(win, "\nSolve for (enter the letter)?: ");
		wscanw(win, "%s", &sf, buf);
		if ( !strncmp(&sf, "E", sizeof(char)) ) {
			SOLVE_FOR;
			wprintw(win, "Distance from the equilibrum position = x = ");
			SW; x = num; num = 0;
			wprintw(win, "x = %f", x);
			wprintw(win, "\nForce constant? k = ");
			SW; k = num; num = 0;
			wprintw(win, "k = %f", k);
			num = (k*x*x)/2;
			wprintw(win, "\nPotential Energy = Ep = %f", num);
		} else if ( !strncmp(&sf, "k", sizeof(char)) ) {
			SOLVE_FOR;
			wprintw(win, "Potential Energy? E = ");
			SW; E = num; num = 0;
			wprintw(win, "E = %f", E);
			wprintw(win, "\nSpring squash? x = ");
			SW; x = num; num = 0;
			wprintw(win, "x = %f", x);
			num = (2*E)/(x*x);
			wprintw(win, "\nForce constant = %f", num);
		} else if ( !strncmp(&sf, "x", sizeof(char)) ) {
			SOLVE_FOR;
			wprintw(win, "Potential Energy? E = ");
			SW; E = num; num = 0;
			wprintw(win, "E = %f", E);
			wprintw(win, "\nForce constant? k = ");
			SW; k = num; num = 0;
			wprintw(win, "k = %f", k);
			num = sqrt((2*E)/k);
			wprintw(win, "\nSprint squash = x = %f", num);
		} else {
			NOPE;
		}
		Continue(win);
    	/* end of elif dilemma ... */
	} else {
		werase(win);
		wrefresh(win);
		wprintw(win, "\nDoesn't exist.");
		Continue(win);
	}
	} 

	dilemma = '0';
	fshmeg = ' ';
}

void mgh_shmeg() {
fshmeg = '1'; }
void ek_shmeg() {
fshmeg = '2'; }
void kx_shmeg() {
fshmeg = '3'; }

void
pick_function_list_eg(int old_choice, int new_choice) {
#define MAX_OPTIONS ((sizeof(flist_energy)/sizeof(flist_energy[0])))
	register size_t f;
	attrset(A_NORMAL);
	refresh();
	move(1, 1);
	clrtobot();
	standout();
	printw("Solve the Problem. Select a formula to use.\n\n");
	standend();
	printw("These formulas are available for use; enter the letter of the variable you want to solve for when asked. Carrying appropriate units is your responsibility!");
	for ( f = 0; f < MAX_OPTIONS; f++ ) {
		mvaddstr(6+f, 7, flist_energy[f].text);
	}
	if (old_choice != (-1))
		mvaddstr(6+old_choice, 7, flist_energy[old_choice].text);
	attrset(A_REVERSE);
	mvaddstr(6+new_shmeg, 7, flist_energy[new_shmeg].text);
	attrset(A_NORMAL);
	mvaddstr(LINES-2, 1, "Use 'j' for down and 'k' for up, Enter to run.");
	refresh();
}

void function_list_eg(WINDOW *win) {
	Lastkey = getch();
	switch(Lastkey)
	{
	case 10: case 13:
	case 'x': case 'X':
	case KEY_ENTER:
		erase();
		refresh();
		(*flist_energy[new_shmeg].function)(win);
		erase();
		pick_function_list_eg(old_shmeg, new_shmeg);
		break;
	case 'k':
	case KEY_UP:
		new_shmeg = (new_shmeg == 0) ? new_shmeg : new_shmeg-1;
		pick_function_list_eg(old_shmeg, new_shmeg);
		break;
	case 'j':
	case KEY_DOWN:
		new_shmeg = (new_shmeg == MAX_OPTIONS - 1) ? new_shmeg : new_shmeg + 1;
		flist:
		pick_function_list_eg(old_shmeg, new_shmeg);
		break;
	case 'q': case 'Q':
		standend();
		endwin();
		exit(0);
		break;
	case 'c': case 'C':
		CALCULATOR;
		goto flist;
	default:
		break;
    }
}


/* EOF */
