/*	$Id: gravitation.c,v 1.6 2001/04/26 03:23:19 pvalchev Exp $	*/

/*
 * Copyright (c) 1998-2001 Peter V. Valchev <pvalchev@gmx.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *    
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission
 *    from the author.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define _PHYZ_GRAVITATION_C

#include "phyz.h"
#include "keys.h"
#include "choice.h"

#ifndef lint
static const char rcsid[] = "$Id: gravitation.c,v 1.6 2001/04/26 03:23:19 pvalchev Exp $";
#endif /* lint */

void k_shmgk ();
void fg_shmgk ();
void g_shmgk ();
void vg_shmgk ();
void tg_shmgk ();
void pick_function_list_g ();
void function_list_g ();

int old_shmgk = -1;
int new_shmgk = 0;

char fshmgk;

#define PI_CHAR 0x1c

CHOICE flist_gravitation[] =
{
	{ "K = R/T", k_shmgk },
	{ "Fg = Gm1m2/R", fg_shmgk },
	{ "g = Gm/R", g_shmgk },
	{ "V = Gm/R", vg_shmgk },
	{ "T = 4R/Gm", tg_shmgk }
};

extern void
gravitation (WINDOW *win) {
	char buf[80];
	char sf;
	float K, R, T, V, m, g = 9.81, G = 0.0000000000667, num, pi = M_PI, F;
	float m1, m2;
	pick_choice(old_choice, new_choice);
	while ( dilemma != '3' && dilemma != '4' ) {
		choice_switch();
	}
	win = newwin(LINES, 0, 0, 0);

	/* Theory */
    if ( dilemma == '3' ) {
	waddstr(win, "\tKEPLER'S LAWS:
A) TERMS:
i) Geocentric model of solar system
   A model of the solar system that places the Earth at the center of the solar system.
ii) Heliocentric model of solar system
   A model of the solar system that places the Sun at the center of the solar system.
iii) Retrogression:
   A planet appears to move backwards in its orbit for a short period of time.

B) Contributions of:
i) Copernicus
   Designed a model of the solar system with the Sun at the center.
ii) Brahe
   Made detailed studies of the motion of stars and planets. Hired Kepler as an assistant.
iii) Galileo
   Deisnged an astronomical telescope to study the stars and planets. He confirmed that Copernicus' model was correct.
iv) Kepler
   Used Brahe's detailed studies to formulate his laws concerning satellite motion around larger bodies.
   ");

	Continue(win);
	werase(win);
	wrefresh(win);
	waddstr(win, "\nC) Kepler's laws

1) elliptical orbits
    The planets move around the sun in elliptical orbits, with the sun at one focus at the ellipses.
2) equal areas
    The straight line joining the sun and a given planet sweeps out equal areas in equal intervals of time.
3) periods
    The square of the period of revolution of a planet about the sun is proportional to the cube of its mean distance from the sun.
4) Kepler's constant K = R/T
    Kepler's constant is same for any satellite (natural or artificial ) that circiles the same planet or star.
    comparison formula for two satellites circling the same planet or star:
	 T(2)    R(2)^
	 ----  =  ----
	 T(1)    R(1)^ 
	");
	Continue(win);
	werase(win);
	wrefresh(win);
	wprintw(win, "\t\nNewton's Universal Gravitation Law

A) Gravitational Force
    Each mass pulls on the other, thus the direction of the Force is inwards towards the center of the mass.
	    Fg = Gm1m2/R
	Where R equals the distance between the centers of mass for each object and G = 6.67e-11 Nm/kg
B) Comparison question
\n1) Changing the distance between the same two masses
	   Fg(1)R(1) = Fg(2)R(2)
2) Changing one of the masses and the distance between the masses:
	   Fg(1)R(1)/m(1) = Fg(2)R(2)/m(2)
C) Weight
    The Earth's gravitational force acting on an object is often called the WEIGHT of the object:
	    WEIGHT = Fg
	");
	Continue(win);
	werase(win);
	wrefresh(win);
	wprintw(win, "\t\nSATELLITES\n
1) Orbit speed is the speed necessary to maintain orbit
the force of gravity provides the centripetal force:
	Fc = Fg
	m(1)V/R = Gm(1)m(2)/R - mass of satellite cancels
	=> V = Gm/R
		where R = radius of ORBIT for satellite
		      m = mass of planet, star, etc.
\n2) PERIOD of satellite
	Use V = Gm/R
	V = d/t, but d = 2R
	V = 2R/T
	V = 4R/T
to get:
	T = 4R/Gm
\n3) Remember the term GEOSYNCHRONIOUS satellite
	A satellite that remains stationary over a point on the EARTH because it has the same period as the Earth.
	");
	Continue(win);
	} 

	/* Solver */
    if ( dilemma == '4' ) { 
	werase(win);
	wrefresh(win);
	*buf = 0;
	num = 0;
	/* XXX: new menu */
	pick_function_list_g(old_choice, new_choice);
	/* listen until : */
	while ( fshmgk != '1' && fshmgk != '2' && fshmgk != '3' && fshmgk != '4' && fshmgk != '5' ) {
		function_list_g();
	}
	echo();
	if ( fshmgk == '1' ) {
		werase(win);
		wrefresh(win);
		waddstr(win, "\nK = R^3/T^2");
		waddstr(win, "\nSolve for (enter the letter)?: ");
		wscanw(win, "%s", &sf, buf);
			if ( !strncmp(&sf, "K", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "Distance? R = ");
				wscanw(win, "%f", &num, buf);
				R = num; num = 0;
				wprintw(win, "R = %f", R);
				wprintw(win, "\nPeriod? T = ");
				wscanw(win, "%f", &num, buf);
				T = num; num = 0;
				wprintw(win, "T = %f", T);
				num = (R*R*R)/(T*T);
				wprintw(win, "\nKepler's const = K = %f", num);
			} else if ( !strncmp(&sf, "T", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "Distance? R = ");
				wscanw(win, "%f", &num, buf);
				R = num; num = 0;
				wprintw(win, "R = %f", R);
				wprintw(win, "\nKepler's const? K = ");
				wscanw(win, "%f", &num, buf);
				K = num; num = 0;
				wprintw(win, "K = %f", K);
				num = sqrt(K/(R*R*R));
				wprintw(win, "\nPeriod = T = %f", num);
			} else if ( !strncmp(&sf, "R", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "Kepler's const? K = ");
				SW; K = num; num = 0;
				wprintw(win, "K = %f", K);
				wprintw(win, "\nPeriod? T = ");
				SW; T = num; num = 0;
				wprintw(win, "T = %f", T);
				num = cbrt(K*T*T);
				wprintw(win, "\nDistance = R = %f", num);
			} else {
				NOPE;
			}
		Continue(win);
	} else if ( fshmgk == '2' ) {
		werase(win);
		wrefresh(win);
		waddstr(win, "\nF = Gm1m2/R");
		waddstr(win, "\nSolve for (enter the letter)?: ");
		wscanw(win, "%s", &sf, buf);
			if ( !strncmp(&sf, "F", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "Distance? R = ");
				SW; R = num; num = 0;
				wprintw(win, "R = %f", R);
				wprintw(win, "\nMass 1 = m1 = ");
				SW; m1 = num; num = 0;
				wprintw(win, "m1 = %f", m1); 
				wprintw(win, "\nMass 2 = m2 = ");
				SW; m2 = num; num = 0;
				wprintw(win, "\nm2 = %f", m2);
				num = (G*m1*m2)/(R*R);
				wprintw(win, "\nGravitational Force = Fg = %f", num);
			} else if ( !strncmp(&sf, "m1", sizeof(char)) || !strncmp(&sf, "m2", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "Distance? R = ");
				SW; R = num; num = 0;
				wprintw(win, "R = %f", R);
				wprintw(win, "\nKnown mass? m = ");
				SW; m = num; num = 0;
				wprintw(win, "m = %f", m);
				wprintw(win, "\nG Force? F = ");
				SW; F = num; num = 0;
				wprintw(win, "\nF = %f", F);
				num = (F*R*R)/(G*m);
				wprintw(win, "\nUnknown mass = m(1/2) = %f", num);
			} else if ( !strncmp(&sf, "R", sizeof(char)) ) {
				SOLVE_FOR;
				wprintw(win, "Gravitational Force? F = ");
				SW; F = num; num = 0;
				wprintw(win, "F = %f", F);
				wprintw(win, "\nMass 1? m1 = ");
				SW; m1 = num; num = 0;
				wprintw(win, "m1 = %f", m1);
				wprintw(win, "\nMass 2? m2 = ");
				SW; m2 = num; num = 0;
				wprintw(win, "\nm2 = %f", m2);
				num = sqrt(F/(G*m1*m2));
				wprintw(win, "\nDistance = R = %f", num);
			} else {
			NOPE;
			}
		Continue(win);
	} else if ( fshmgk == '3' ) {
		werase(win); 
		wrefresh(win);
		waddstr(win, "\ng = Gm/R");
		waddstr(win, "\nSolve for (enter the letter)?: ");
		wscanw(win, "%s", &sf, buf);
		if ( !strncmp(&sf, "g", sizeof(char)) ) {
			SOLVE_FOR;
			wprintw(win, "Mass? m = ");
			SW; m = num; num = 0;
			wprintw(win, "m = %f", m);
			wprintw(win, "\nDistance? R = ");
			SW; R = num; num = 0;
			wprintw(win, "R = %f", R);
			num = (G*m)/R;
			wprintw(win, "\nGravitational field strenght = g = %f", num);
		} else if ( !strncmp(&sf, "m", sizeof(char)) ) {
			SOLVE_FOR;
			wprintw(win, "Grav. field strenght? g = ");
			SW; g = num; num = 0;
			wprintw(win, "g = %f", g);
			wprintw(win, "\nDistance? R = ");
			SW; R = num; num = 0;
			wprintw(win, "R = %f", R);
			num = (g*R*R)/G;
			wprintw(win, "\nMass = m = %f", num);
		} else if ( !strncmp(&sf, "R", sizeof(char)) ) {
			SOLVE_FOR;
			wprintw(win, "Grav. field strenght? g = ");
			SW; g = num; num = 0;
			wprintw(win, "g = %f", g);
			wprintw(win, "\nMass? m = ");
			SW; m = num; num = 0;
			wprintw(win, "m = %f", m);
			num = sqrt(g/(G*m));
			wprintw(win, "\nDistance = R = %f", num);
		} else {
			NOPE;
		}
	Continue(win);
	} else if ( fshmgk == '4' ) {
		werase(win); 
		wrefresh(win);
		waddstr(win, "\nV = Gm/R");
		waddstr(win, "\nSolve for (enter the letter)?: ");
		wscanw(win, "%s", &sf, buf);
		if ( !strncmp(&sf, "V", sizeof(char)) ) {
			SOLVE_FOR;
			wprintw(win, "Mass? m = ");
			SW; m = num; num = 0;
			wprintw(win, "m = %f", m);
			wprintw(win, "\nOrbital radius? R = ");
			SW; R = num; num = 0;
			wprintw(win, "R = %f", R);
			num = sqrt((G*m)/R);
			wprintw(win, "\nSpeed = V = %f", num);
		} else if ( !strncmp(&sf, "m", sizeof(char)) ) {
			SOLVE_FOR;
			wprintw(win, "Orbital radius? R = ");
			SW; R = num; num = 0;
			wprintw(win, "R = %f", R);
			wprintw(win, "\nSpeed = V = ");
			SW; V = num; num = 0;
			wprintw(win, "V = %f", V);
			num = (V*V*R)/G;
			wprintw(win, "\nMass = m = %f", num);
		} else if ( !strncmp(&sf, "R", sizeof(char)) ) {
			SOLVE_FOR;
			wprintw(win, "Mass? m = ");
			SW; m = num; num = 0;
			wprintw(win, "m = %f", m);
			wprintw(win, "\nSpeed? V = ");
			SW; V = num; num = 0;
			wprintw(win, "V = %f", V);
			num = (G*m)/(V*V);
			wprintw(win, "\nOrbital radius = R = %f", num);
		} else {
			NOPE;
		}
		Continue(win);
	} else if ( fshmgk == '5' ) {
		werase(win); 
		wrefresh(win);
		waddstr(win, "\nT = 4R^3/Gm");
		waddstr(win, "\nSolve for (enter the letter)?: ");
		wscanw(win, "%s", &sf, buf);
		if ( !strncmp(&sf, "T", sizeof(char)) ) {
			SOLVE_FOR;
			wprintw(win, "Orbital radius? R = ");
			SW; R = num; num = 0;
			wprintw(win, "R = %f", R);
			wprintw(win, "\nMass? m = ");
			SW; m = num; num = 0;
			wprintw(win, "m = %f", m);
			num = sqrt((4*pi*pi*R*R*R)/(G*m));
			wprintw(win, "\nPeriod = T = %f", num);
		} else if ( !strncmp(&sf, "R", sizeof(char)) ) {
			SOLVE_FOR;
			wprintw(win, "Period? T = ");
			SW; T = num; num = 0;
			wprintw(win, "T = %f", T);
			wprintw(win, "\nMass? m = ");
			SW; m = num; num = 0;
			wprintw(win, "m = %f", m);
			num = cbrt((T*T*G*m)/(4*pi*pi));
			wprintw(win, "\nOrbital radius = R = %f", num);
		} else if ( !strncmp(&sf, "m", sizeof(char)) ) {
			SOLVE_FOR;
			wprintw(win, "Period? T = ");
			SW; T = num; num = 0;
			wprintw(win, "T = %f", T);
			wprintw(win, "\nOrbital radius? R = ");
			SW; R = num; num = 0;
			wprintw(win, "R = %f", R);
			num = (4*pi*pi*R*R*R)/(G*T*T);
			wprintw(win, "\nMass = m = %f", num);
		} else {
			NOPE;
		}
		Continue(win);
    	/* end of elif dilemma ... */
	} else {
		werase(win);
		wrefresh(win);
		wprintw(win, "\nDoesn't exist.");
		Continue(win);
	}
	} 

	dilemma = '0';
	fshmgk = '0';
}

void k_shmgk () { 
fshmgk = '1'; }
void fg_shmgk () {
fshmgk = '2'; }
void g_shmgk () {
fshmgk = '3'; }
void vg_shmgk () {
fshmgk = '4'; }
void tg_shmgk () {
fshmgk = '5'; }

void
pick_function_list_g(int old_choice, int new_choice) {
#define MAX_OPTIONS ((sizeof(flist_gravitation)/sizeof(flist_gravitation[0])))
	register size_t f;
	attrset(A_NORMAL);
	refresh();
	move(1, 1);
	clrtobot();
	standout();
	printw("Solve the Problem. Select a formula to use.\n\n");
	standend();
	printw("These formulas are available for use; enter the letter of the variable you want to solve for when asked. Carrying appropriate units is your responsibility!");
	for ( f = 0; f < MAX_OPTIONS; f++ ) {
		mvaddstr(6+f, 7, flist_gravitation[f].text);
	}
	if (old_choice != (-1))
		mvaddstr(6+old_choice, 7, flist_gravitation[old_choice].text);
	attrset(A_REVERSE);
	mvaddstr(6+new_shmgk, 7, flist_gravitation[new_shmgk].text);
	attrset(A_NORMAL);
	mvaddstr(LINES-2, 1, "Use 'j' for down and 'k' for up, Enter to run.");
	refresh();
}

void function_list_g(WINDOW *win) {
	Lastkey = getch();
	switch(Lastkey)
	{
	case 10: case 13:
	case 'x': case 'X':
	case KEY_ENTER:
		erase();
		refresh();
		(*flist_gravitation[new_shmgk].function)(win);
		erase();
		pick_function_list_g(old_shmgk, new_shmgk);
		break;
	case 'k':
	case KEY_UP:
		new_shmgk = (new_shmgk == 0) ? new_shmgk : new_shmgk-1;
		pick_function_list_g(old_shmgk, new_shmgk);
		break;
	case 'j':
	case KEY_DOWN:
		new_shmgk = (new_shmgk == MAX_OPTIONS - 1) ? new_shmgk : new_shmgk + 1;
		flist:
		pick_function_list_g(old_shmgk, new_shmgk);
		break;
	case 'q': case 'Q':
		standend();
		endwin();
		exit(0);
		break;
	case 'c': case 'C':
		CALCULATOR;
		goto flist;
	default:
		break;
    }
}


/* EOF */
