/*	$Id: phyz.c,v 1.47 2001/04/26 03:23:19 pvalchev Exp $	*/

/*
 * Copyright (c) 1998-2001 Peter V. Valchev <pvalchev@gmx.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *    
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission
 *    from the author.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define _PHYZ_PHYZ_C

#include "phyz.h"
#include "choice.h"
#include "keys.h"
#include <string.h>
#include <unistd.h>

int i;
int old_option = -1, new_option = 0;
int main ();

#ifndef lint
static const char rcsid[] = "$Id: phyz.c,v 1.47 2001/04/26 03:23:19 pvalchev Exp $";
#endif /* lint */

/* Function declaration */
extern void circular_motion (WINDOW *);
extern void projectile_motion (WINDOW *);
extern void gravitation (WINDOW *);
extern void electricity (WINDOW *);
extern void energy (WINDOW *);

struct commands {
	char *text;
	void (*function)(WINDOW *);
};

typedef struct commands COMMAND;

int done = 0;

const COMMAND command[] =
{
	{ "Circular Motion", circular_motion },
	{ "Projectile Motion + Distance/Speed formulas", projectile_motion },
	{ "Gravitation", gravitation },
	{ "Electricity and Magnetism", electricity },
	{ "Potential and Kinetic Energy", energy }
};

#define MAX_OPTIONS ((sizeof(command)/sizeof(command[0])))

int width, height;

int
key_switch(WINDOW *win) {
  Lastkey = getch();
  switch (Lastkey)
  {
    case 10: case 13: 
    case 'x': case 'X':	/* x/X would stand for execute */
    case KEY_ENTER:
        erase();
        refresh();
        (*command[new_option].function)(win);
        erase();
        main_screen(old_option, new_option);
        break;   
    case 'k':
    case KEY_UP:
        new_option = (new_option == 0) ? new_option : new_option-1;
        main_screen(old_option, new_option);
        break;
    case 'j':
    case KEY_DOWN:
        new_option = (new_option == MAX_OPTIONS - 1) ? new_option : new_option+1;
        main_screen(old_option, new_option);
        break;
    case 'c':
	CALCULATOR;
        main_screen(old_option, new_option);
	break;
    case KEY_LEFT:
    case '\002':
      return OP_EDITOR_BACKWARD_CHAR;
    case KEY_RIGHT:
    case '\006':
      return OP_EDITOR_FORWARD_CHAR;
    case '\010':
    case KEY_BACKSPACE:
      return OP_EDITOR_BACKSPACE;
    case '\004':
    case KEY_DC:
    case 127:
      return OP_EDITOR_DELETE_CHAR;
    case '\001':
      return OP_EDITOR_BOL;
    case '\005':
      return OP_EDITOR_EOL;
    case '\025':
      return OP_EDITOR_KILL_LINE;
    case '\013':
      return OP_EDITOR_KILL_EOL;
    case '\027':
      return OP_EDITOR_KILL_WORD;
    case '\007':
    case -1:
      return (-1);
    case 'v': case 'V':
      move(0, 50);
      printw("Physics Solving Toolkit %s\n", version);
      refresh();
      break;
    case 'q': case 'Q':
      standend();
      done = 1;
      break;
    default:
      break;
  }
  return 0;
}

void Continue (WINDOW *win) {
	wmove(win, 10, 1);
	wclrtoeol(win);
	mvwaddstr(win, LINES-2, 1, " Press anything to continue");
	wrefresh(win);
	raw();
	switch ( wgetch(win) ) {
	case 'c': case 'C':
		CALCULATOR;
		break;
	default:
		break;
	}

}

int
main (argc, argv)
	int argc;
	char **argv;
{
	width = COLS;
	height = LINES;
	while (--argc) {
		if (!strcmp(argv[argc], "-V") || !strcmp(argv[argc], "-v")) {
		(void) fprintf(stderr, "Phyz version %s\n", version);
		exit(0);
		} else if (!strcmp(argv[argc], "-h") || !strcmp(argv[argc], "--help") || !strcmp(argv[argc], "-help")) {
		(void) fprintf(stderr, "Physics Solving Toolkit %s.\n", version);
		(void) fprintf(stderr, "Copyright (c) 2001 Peter Valchev.\n");
		(void) fprintf(stderr, "Usage:\n");
		(void) fprintf(stderr, "    -V - version\n");
		(void) fprintf(stderr, "    -h - usage help\n");
		(void) fprintf(stderr, "    -license - show license information\n");
		exit(0);
		} else if ( !strcmp(argv[argc], "-license") || !strcmp(argv[argc], "--license") || !strcmp(argv[argc], "-l") ) {
		(void) fprintf(stderr, "Copyright (c) 2001 Peter V. Valchev.\n");
		(void) fprintf(stderr, "All rights reserved.\n");
		(void) fprintf(stderr, "See the file LICENSE provided with the package for more information.\n");
		exit(0);
		} else {
		usage();
		}
	}
	initscr();
	cbreak(); noecho();
	keypad(stdscr, TRUE);
	erase();
	main_screen(old_option, new_option);
	while (!done) {
		dokey();
	}
	endwin();
	exit(0);
}

int
usage() 
{
	(void) fprintf(stderr, "usage: phyz [-h] [-V] [-l]\n");
	exit(1);
	return(0);
}

int
main_screen(int old_option, int new_option, WINDOW *win) {
	register size_t i;
	win = newwin(LINES, 0, 0, 0);
	if (win == NULL ) {
		endwin();
		return 0;	/* something's wrong ... */
	}
	attrset(A_NORMAL);
	refresh();
	move(0, 0);
	clrtobot();
	standout();
	printw("Phyz %s: Press Q to Quit", version);
	standend();
	for ( i = 0; i < MAX_OPTIONS; i++ ) {
		mvaddstr(5+i, 10, command[i].text);
	}
	if (old_option != (-1))
		mvaddstr(5+old_option, 10, command[old_option].text);
	attrset(A_REVERSE);
	mvaddstr(5+new_option, 10, command[new_option].text);
	attrset(A_NORMAL);
	mvaddstr(LINES-2, 1, "Use 'j' for down and 'k' for up, Enter to run, Q to quit");
	refresh();
	return(0);
}

/* EOF */
